==> rospell-2007.02.rc1.tar.gz: rospell-2007.02.rc1/src/rospell/qtools.cpp <==
//****************************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************

#include "qwerty.h"
#include "briefedit.h"
#include "diffdialog.h"
#include "beautydialog.h"
#include "busycursor.h"
#include "makedialog.h"
#include "qtutils.h"
#include "global.h"
#include "filevisitor.h"
#include "progressbar.h"
#include "qtutils.h"
#include <qapplication.h>
#include <qcursor.h>
#include <qfileinfo.h>

//****************************************************************************************
// local functions
//****************************************************************************************
static void cleanTags(char *str, char *tag1, char *tag2) {
	char *ptr1 = strstr(str, tag1);
	if (ptr1 == 0)
		return;
	char *ptr3 = strstr(ptr1, tag2);
	if (ptr3 == 0)
		return;
	ptr3 += strlen(tag2);

	if (ptr1 < ptr3) {
		while(ptr1 != ptr3) {
			*ptr1 = ' ';
			ptr1++;
		}
	}
}

static char *stripDexonline(char *str) {
	ASSERT(str != 0);
	if (str == 0)
		return 0;

	char *ptr1 = UTILS::buffer_sed(str, "&#x103;", "ă");
	delete[] str;
	char *ptr2 = UTILS::buffer_sed(ptr1, "&#xee;", "î");
	delete[] ptr1;
	ptr1 = UTILS::buffer_sed(ptr2, "&#xe2;", "â");
	delete[] ptr2;
	ptr2 = UTILS::buffer_sed(ptr1, "&#x2013;", "-");
	delete[] ptr1;
	ptr1 = UTILS::buffer_sed(ptr2, "&#xcd;", "I");
	delete[] ptr2;
	ptr2 = UTILS::buffer_sed(ptr1, "&#x102;", "Ă");
	delete[] ptr1;
	ptr1 = UTILS::buffer_sed(ptr2, "&#xc2;", "Â");
	delete[] ptr2;
	ptr2 = UTILS::buffer_sed(ptr1, "&#xc9;", "E");
	delete[] ptr1;
	ptr1 = UTILS::buffer_sed(ptr2, "&#xce;", "Î");
	delete[] ptr2;
	ptr2 = UTILS::buffer_sed(ptr1, "&#xd3;", "O");
	delete[] ptr1;
	ptr1 = UTILS::buffer_sed(ptr2, "&#xda;", "U");
	delete[] ptr2;
	ptr2 = UTILS::buffer_sed(ptr1, "&#xc1;", "A");
	delete[] ptr1;

	cleanTags(ptr2, "<script", "</script>");
	cleanTags(ptr2, "<script", "</script>");
	cleanTags(ptr2, "<script", "</script>");
	cleanTags(ptr2, "<script", "</script>");
	cleanTags(ptr2, "<script", "</script>");
	cleanTags(ptr2, "<script", "</script>");
	cleanTags(ptr2, "<noscript", "</noscript>");
	cleanTags(ptr2, "<form", "</form>");
	return ptr2;
}

//****************************************************************************************
// editor tools menu implementation
//****************************************************************************************
void Editor::recordAction() {
	e_->recordAction();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	cursorPosChanged(para, index);
}

void Editor::playbackAction() {
	e_->playbackAction();
}

void Editor::unixCleanup() {
	QString t1 = e_->text();
	QString t2 = QTUTILS::unixEol(t1);
	e_->setText(t2);
}



void Editor::uppercase() {
	QString text1 = e_->markedText();
	QString text2 = text1.upper();
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	e_->insertAt(text2, para, index, false);
}

void Editor::lowercase() {
	QString text1 = e_->markedText();
	QString text2 = text1.lower();
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	e_->insertAt(text2, para, index, false);
}



void Editor::dexonline() {
	// check for wget presence
	if (!Global::checkWhich("wget"))
		return;
	QString sword = e_->markedText();
	if (sword.isNull()) {
		msgScreen("Please select a word first!");
		LOG("dexonline no word");
		return;
	}
	Q3CString utext = sword.utf8();
	const char *cstr = utext;
	LOG("Editor::dexonline  %s", cstr);
	msgScreen("Loading...");
	QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
	char cmd[1000];

	Global *glb = Global::instance();
	if (glb->lang_ro)
		snprintf(cmd, 1000, "wget --timeout=60 --tries=1 -q --output-document=- http://dexonline.ro/search.php?cuv=%s&source=", cstr);
	else if (glb->lang_en)
//		snprintf(cmd, 1000, "wget --timeout=60 --tries=1 -q --output-document=- http://machaut.uchicago.edu/cgi-bin/WEBSTER.sh?WORD=%s", cstr);
		snprintf(cmd, 1000, "wget --timeout=60 --tries=1 -q --output-document=- http://www.bennetyee.org/http_webster.cgi?isindex=%s&method=exact", cstr);

	else
		return;

	char *www_content = UTILS::run_prg_malloc(cmd);
	msgScreen(0);
	QApplication::restoreOverrideCursor();
	if (www_content != 0) {
		char *newcontent = stripDexonline(www_content);
		if (newcontent == 0) {
			delete[] www_content;
			return;
		}
		LOG("******\n%s\n***************\n", newcontent);
		if (glb->lang_ro)
			QMessageBox::about(0, "http://dexonline.ro", QString::fromUtf8(newcontent));
		else if (glb->lang_en)
//			QMessageBox::about(0, "http://machaut.uchicago.edu/orgs/ARTFL/", QString::fromUtf8(newcontent));
			QMessageBox::about(0, "http://www.bennetyee.org", QString::fromUtf8(newcontent));
		delete[] newcontent;
	}
}

void Editor::dirMerge() {
	cleanMWords();

	DirDiffDialog *dd = Global::instance()->ddd;
	if (dd == 0) {
		dd = new DirDiffDialog();
		dd->setCaption("Enter directories");
		Global::instance()->ddd = dd;
	}

	if (!dd->exec()) {
		delete dd;
		return;
	}

	QString file1 = dd->getFile1();
	QString file2 = dd->getFile2();
	QString fpattern = dd->getFpattern();
	QString edirectory = dd->getEdirectory();

	if (file1.isNull() || file1.isEmpty() || file1.isNull() || file2.isEmpty()) {
		QMessageBox::warning(0,"Warning", "Please chose two directories to merge");
		return;
	}

	ProgressBar *progress = new ProgressBar(this, "read merge directories");
	progress->show();
	progress->start(10000);
	FVBase fvb1(file1, fpattern, edirectory, true);
	FVisitorDirfilesFull visitor1(file1);
	QString txt1 = fvb1.walk(&visitor1);

	FVBase fvb2(file2, fpattern, edirectory, true);
	FVisitorDirfilesFull visitor2(file2);
	QString txt2 = fvb2.walk(&visitor2);
	delete progress;

	progress = new ProgressBar(this, "process merge directories");
	progress->show();
	QFont fnt = e_->font();
	DirMergeWorksDialog *wd = new DirMergeWorksDialog(fnt, file1, file2, txt1, txt2, progress);
	delete progress;
	wd->setCaption("dir diff");
	wd->exec();
	delete wd;
}

void Editor::diff() {
	cleanMWords();
	DiffDialog *dd = new DiffDialog();
	dd->setCaption("Enter files");
	if (!dd->exec()) {
		delete dd;
		return;
	}

	QString file1 = dd->getFile1();
	QString file2 = dd->getFile2();
	delete dd;
LOG("diff file1: %s", (const char *) file1);
LOG("diff file2: %s", (const char *) file2);

	QFont fnt = e_->font();
	DiffWorksDialog *wd = new DiffWorksDialog(fnt, file1, file2);
	wd->setCaption("diff");
	wd->exec();
	delete wd;
}

void Editor::merge() {
	cleanMWords();
	DiffDialog *dd = new DiffDialog();
	dd->setCaption("Enter files");
	if (!dd->exec()) {
		delete dd;
		return;
	}

	QString file1 = dd->getFile1();
	QString file2 = dd->getFile2();
	delete dd;
LOG("merge file1: %s", (const char *) file1);
LOG("merge file2: %s", (const char *) file2);

	QFont fnt = e_->font();
	MergeWorksDialog *wd = new MergeWorksDialog(fnt, file1, file2);
	wd->setCaption("diff");
	wd->exec();
	delete wd;
}

void Editor::sessionDiff() {
	cleanMWords();

	// save current buffer
//	save();
	QString file1 = caption();
	if (file1.isNull() || file1.isEmpty() || file1 == "*") {
		QMessageBox::warning(0,"Warning", "Session diff not supported on new files");
		return;
	}
	if (!save()) {
		QMessageBox::warning(0,"Error", "Cannot save the current file.");
		return;
	}
	file1 = caption();

	// get file from backup
	QFileInfo qf(file1);
	char *f2 = Global::getBackupFname((const char *) qf.fileName());
	QString file2 = QString(f2);
	delete[] f2;
LOG("session diff file1: %s", (const char *) file1);
LOG("session diff file2: %s", (const char *) file2);

	QFont fnt = e_->font();
	if (file1.isEmpty() || file1.isNull() || file2.isEmpty() || file2.isNull())
		return;

	DiffWorksDialog *wd = new DiffWorksDialog(fnt, file2, file1);
	wd->setCaption("session diff");
	wd->exec();
	delete wd;
}

void Editor::beautify() {
	BeautyDialog *be = new BeautyDialog();

	be->setCaption("Code beautifier options");
	if (!be->exec()) {
		delete be;
		return;
	}

	QString file = caption();
	if (file.isNull() || file.isEmpty() || file == "*") {
		QMessageBox::warning(0,"Warning", "First you will need to save the file");
		return;
	}

	// save file
	save();
	file = caption();

	tw_user_->setText("");
	BCursor busy;
	msgScreen("kloc, please wait...");
	QString qs = be->getCmd(file);
	tw_user_->setText(qs);
	tw_->setCurrentPage(2); //lucian: hardcoded
	char *result = UTILS::run_prg_malloc((const char *) qs);
	if (result == 0) {
		QMessageBox::warning(0,"Warning", "Failed!");
		// force a resave of the file
		save();
		return;
	}

	e_->setText(result);
	e_->setFocus();
	delete[] result;
}

void Editor::kloc() {
	tw_user_->setText("");
	msgScreen("kloc, please wait...");
	BCursor busy;

	char *result = UTILS::run_prg_malloc("rokloc -d -s");
	tw_user_->setText(result);
	tw_->setCurrentPage(2); //lucian: hardcoded
	msgScreen("done", 2000);
	delete[] result;
}

void Editor::make() {
	tw_user_->setText("");
	msgScreen("make, please wait...");

	QString cmd;
	if (Global::instance()->make_cmd_.isEmpty()) {
		MakeDialog *d = new MakeDialog();
		d->setCaption("make");
		if (!d->exec()) {
			delete d;
			return;
		}
		cmd = d->getCmd();
		if (d->getDontShow())
			Global::instance()->make_cmd_ = cmd;
		delete d;
	}
	else
		cmd = Global::instance()->make_cmd_;

	BCursor busy;
	QTUTILS::run_program(cmd, tw_user_, "make done");
	tw_->setCurrentPage(2); //lucian: hardcoded
	msgScreen("done", 2000);
}

void Editor::stripTrailingSpaces() {
	BCursor busy;
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString intext = e_->text();
	QString newtext = QTUTILS::stripTrailingSpaces(intext);
	e_->setText(newtext);
	e_->setCursorPosition(para, index, false);
}

void Editor::cedilla2commabelow() {
	BCursor busy;
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString intext = e_->text();
	QString newtext = QTUTILS::cedilla2commabelow(intext);
	e_->setText(newtext);
	e_->setCursorPosition(para, index, false);
}

void Editor::commabelow2cedilla() {
	BCursor busy;
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString intext = e_->text();
	QString newtext = QTUTILS::commabelow2cedilla(intext);
	e_->setText(newtext);
	e_->setCursorPosition(para, index, false);
}

void Editor::toggleSpeller() {
	if (show_speller_) {
		mwords_->hide();
		show_speller_ = false;
	}
	else {
		mwords_->show();
		show_speller_ = true;
	}
}

==> rospell-2009.00-rc2.tar.gz: rospell-2009.00-rc2/src/rospell/qtools.cpp <==
//****************************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************

#include "qwerty.h"
#include "briefedit.h"
#include "diffdialog.h"
#include "beautydialog.h"
#include "busycursor.h"
#include "programdialog.h"
#include "qtutils.h"
#include "global.h"
#include "filevisitor.h"
#include "progressbar.h"
#include "qtutils.h"
#include "gdb.h"
#include "gdbedit.h"
#include <qapplication.h>
#include <qcursor.h>
#include <qfileinfo.h>

//****************************************************************************************
// local functions
//****************************************************************************************
//lucian: duplicated code in qhelp.cpp
static void show_help_window(QFont f, QString title, QString content) {
	QTextEdit *qb = new QTextEdit(0, "dictionary");
	qb->setText(content);
	qb->setReadOnly(true);
//	qb->scrollToAnchor(anchor);
	qb->setCaption(title); //lucian: tr doesn't work
	qb->setPaletteBackgroundColor(Global::instance()->c_background);
	qb->setPaletteForegroundColor(Global::instance()->c_foreground);
	QPalette qp = qb->palette();
	qp.setColor(QPalette::Active, QColorGroup::Highlight, Global::instance()->c_highlight);
	qp.setColor(QPalette::Inactive, QColorGroup::Highlight, Global::instance()->c_highlight);
	qb->setPalette(qp);
	qb->setFont(f);
	qb->resize(400, 300);
	qb->show();
}

static void cleanTags(char *str, const char *tag1, const char *tag2) {
	char *ptr1 = strstr(str, tag1);
	if (ptr1 == 0)
		return;
	char *ptr3 = strstr(ptr1, tag2);
	if (ptr3 == 0)
		return;
	ptr3 += strlen(tag2);

	if (ptr1 < ptr3) {
		while(ptr1 != ptr3) {
			*ptr1 = ' ';
			ptr1++;
		}
	}
}

static char *stripDexonline(char *str) {
	ASSERT(str != 0);
	if (str == 0)
		return 0;

	char *ptr1 = UTILS::buffer_sed(str, "&#x103;", "ă");
	delete[] str;
	char *ptr2 = UTILS::buffer_sed(ptr1, "&#xee;", "î");
	delete[] ptr1;
	ptr1 = UTILS::buffer_sed(ptr2, "&#xe2;", "â");
	delete[] ptr2;
	ptr2 = UTILS::buffer_sed(ptr1, "&#x2013;", "-");
	delete[] ptr1;
	ptr1 = UTILS::buffer_sed(ptr2, "&#xcd;", "I");
	delete[] ptr2;
	ptr2 = UTILS::buffer_sed(ptr1, "&#x102;", "Ă");
	delete[] ptr1;
	ptr1 = UTILS::buffer_sed(ptr2, "&#xc2;", "Â");
	delete[] ptr2;
	ptr2 = UTILS::buffer_sed(ptr1, "&#xc9;", "E");
	delete[] ptr1;
	ptr1 = UTILS::buffer_sed(ptr2, "&#xce;", "Î");
	delete[] ptr2;
	ptr2 = UTILS::buffer_sed(ptr1, "&#xd3;", "O");
	delete[] ptr1;
	ptr1 = UTILS::buffer_sed(ptr2, "&#xda;", "U");
	delete[] ptr2;
	ptr2 = UTILS::buffer_sed(ptr1, "&#xc1;", "A");
	delete[] ptr1;

	cleanTags(ptr2, "<script", "</script>");
	cleanTags(ptr2, "<script", "</script>");
	cleanTags(ptr2, "<script", "</script>");
	cleanTags(ptr2, "<script", "</script>");
	cleanTags(ptr2, "<script", "</script>");
	cleanTags(ptr2, "<script", "</script>");
	cleanTags(ptr2, "<noscript", "</noscript>");
	cleanTags(ptr2, "<form", "</form>");
	return ptr2;
}

//****************************************************************************************
// editor tools menu implementation
//****************************************************************************************
void Editor::recordAction() {
	e_->recordAction();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	cursorPosChanged(para, index);
}

void Editor::playbackAction() {
	e_->playbackAction();
}

void Editor::unixCleanup() {
	QString t1 = e_->text();
	QString t2 = QTUTILS::unixEol(t1);
	e_->setText(t2);
}



void Editor::uppercase() {
	QString text1 = e_->markedText();
	QString text2 = text1.upper();
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	e_->insertAt(text2, para, index, false);
}

void Editor::lowercase() {
	QString text1 = e_->markedText();
	QString text2 = text1.lower();
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	e_->insertAt(text2, para, index, false);
}



void Editor::dexonline() {
#ifndef ROSPELL_NO_WGET
	// check for wget presence
	if (!Global::checkWhich("wget"))
		return;
	QString sword = e_->markedText();
	if (sword.isNull()) {
		// extract word at cursor
		// lucian: the code is replicated in briefedit.cpp (quicksearch) and qwerty.cpp (ctags)
		int para;
		int pos;
		e_->getCursorPosition(&para, &pos);
		// lucian: some code here is duplicated from qwerty.cpp void Editor::cursorPosChanged(int para, int pos) - to fix!

		QString line = e_->textLine(para);
		QChar ch = line.at(pos);
		if (ch.isLetterOrNumber() || ch == '_') {
			// extract word
			int xstart = pos;
			while (xstart >= 0 && (line.at(xstart).isLetterOrNumber() || line.at(xstart) == '_'))
				xstart--;
			if (xstart != 0)
				xstart++;

			int xend = pos;
			while (line.at(xend).isLetterOrNumber() || line.at(xend) == '_')
				xend++;

			QString ctagword = line.mid(xstart, xend - xstart);
			if (ctagword.startsWith("\t")) // lucian: some problems with the first letter when is at the beginning of the line! Below is just a hack!
				ctagword = ctagword.mid(1);
			if (ctagword.startsWith("\\"))
				ctagword = ctagword.mid(1);
			// eliminate the bookmark character
			if (QTUTILS::hasBookmarkers(ctagword))
				ctagword = ctagword.mid(1);
			sword = ctagword;
		}
	}

	if (sword.isNull()) {
		msgScreen(tr("Please select a word first!"));
		LOG("dexonline no word");
		return;
	}

	Q3CString utext = sword.utf8();
	const char *cstr = utext;
	LOG("Editor::dexonline  %s", cstr);
	msgScreen(tr("Loading..."));
	QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
	char cmd[1000];
	Global *glb = Global::instance();
	if (glb->lang == LANG_RO_CEDILLA || glb->lang == LANG_RO_COMMABELOW)
		snprintf(cmd, 1000, "wget --timeout=60 --tries=1 -q --output-document=- http://dexonline.ro/search.php?cuv=%s&source=", cstr);
	else if (glb->lang == LANG_EN_US)
//		snprintf(cmd, 1000, "wget --timeout=60 --tries=1 -q --output-document=- http://machaut.uchicago.edu/cgi-bin/WEBSTER.sh?WORD=%s", cstr);
		snprintf(cmd, 1000, "wget --timeout=60 --tries=1 -q --output-document=- http://www.bennetyee.org/http_webster.cgi?isindex=%s&method=exact", cstr);

	else {
		msgScreen(tr("Operation not supported"), 5000);
		return;
	}

	char *www_content = UTILS::run_prg_malloc(cmd);
	msgScreen(0);
	QApplication::restoreOverrideCursor();

	if (www_content == 0) {
		msgScreen(tr("Cannot access dictionary"), 5000);
		return;
	}
	if (www_content != 0) {
		char *newcontent = stripDexonline(www_content);
		if (newcontent == 0) {
			msgScreen(tr("Cannot access dictionary"), 5000);
			delete[] www_content;
			return;
		}
		LOG("******\n%s\n***************\n", newcontent);
		if (glb->lang == LANG_RO_CEDILLA || glb->lang == LANG_RO_COMMABELOW) {
			// convert to commabelow
			QString qs = QString::fromUtf8(newcontent);
			QString newtext = QTUTILS::cedilla2commabelow(qs);
			show_help_window(e_->font(), "http://dexonline.ro", newtext);
//			QMessageBox::about(0, "http://dexonline.ro", QString::fromUtf8(newcontent));
		}
		else if (glb->lang == LANG_EN_US)
//			QMessageBox::about(0, "http://machaut.uchicago.edu/orgs/ARTFL/", QString::fromUtf8(newcontent));
//			QMessageBox::about(0, "http://www.bennetyee.org", QString::fromUtf8(newcontent));
			show_help_window(e_->font(), "http://www.bennetyee.org", QString::fromUtf8(newcontent));
		delete[] newcontent;
	}
#endif
}

void Editor::dirMerge() {
#ifndef ROSPELL_NO_DIFF
	cleanMWords();

	DirDiffDialog *dd = Global::instance()->ddd;
	if (dd == 0) {
		dd = new DirDiffDialog();
		dd->setCaption(tr("dir merge"));
		Global::instance()->ddd = dd;
	}

	if (!dd->exec()) {
		delete dd;
		Global::instance()->ddd = 0;
		return;
	}

	QString file1 = dd->getFile1();
	QString file2 = dd->getFile2();
	QString fpattern = dd->getFpattern();
	QString edirectory = dd->getEdirectory();

	if (file1.isNull() || file1.isEmpty() || file2.isNull() || file2.isEmpty()) {
		QMessageBox::warning(0,tr("Warning"), tr("Please chose two directories to merge"));
		return;
	}

	ASSERT(qvb_ != 0);
	ProgressBar *progress = new ProgressBar(qvb_, tr("Read merge directories"));
	progress->show();
	progress->start(10000);
	FVBase fvb1(file1, fpattern, edirectory, true);
	FVisitorDirfilesFull visitor1(file1);
	QString txt1 = fvb1.walk(&visitor1);

	FVBase fvb2(file2, fpattern, edirectory, true);
	FVisitorDirfilesFull visitor2(file2);
	QString txt2 = fvb2.walk(&visitor2);
	delete progress;

	ASSERT(qvb_ != 0)
	progress = new ProgressBar(qvb_, tr("Process merge directories"));
	progress->show();
	QFont fnt = e_->font();
	DirMergeWorksDialog *wd = new DirMergeWorksDialog(fnt, file1, file2, txt1, txt2, progress);
	delete progress;
	wd->setCaption(tr("dir diff"));
	wd->exec();
	delete wd;
#endif
}

void Editor::diff() {
#ifndef ROSPELL_NO_DIFF
	cleanMWords();
	DiffDialog *dd = new DiffDialog();
	dd->setCaption(tr("diff"));
	if (!dd->exec()) {
		delete dd;
		return;
	}

	QString file1 = dd->getFile1();
	QString file2 = dd->getFile2();
	delete dd;
LOG("diff file1: %s", (const char *) file1);
LOG("diff file2: %s", (const char *) file2);

	if (file1.isNull() || file1.isEmpty() || file2.isNull() || file2.isEmpty()) {
		QMessageBox::warning(0,tr("Warning"), tr("Please chose two files for diff operation"));
		return;
	}

	QFont fnt = e_->font();
	DiffWorksDialog *wd = new DiffWorksDialog(fnt, file1, file2);
	wd->setCaption("diff");
	wd->exec();
	delete wd;
#endif
}

void Editor::merge() {
#ifndef ROSPELL_NO_DIFF
	cleanMWords();
	DiffDialog *dd = new DiffDialog();
	dd->setCaption(tr("merge"));
	if (!dd->exec()) {
		delete dd;
		return;
	}

	QString file1 = dd->getFile1();
	QString file2 = dd->getFile2();
	delete dd;
LOG("merge file1: %s", (const char *) file1);
LOG("merge file2: %s", (const char *) file2);
	if (file1.isNull() || file1.isEmpty() || file2.isNull() || file2.isEmpty()) {
		QMessageBox::warning(0,tr("Warning"), tr("Please chose two files to merge"));
		return;
	}

	QFont fnt = e_->font();
	MergeWorksDialog *wd = new MergeWorksDialog(fnt, file1, file2);
	wd->setCaption(tr("merge"));
	wd->exec();
	delete wd;
#endif
}

void Editor::sessionDiff() {
#ifndef ROSPELL_NO_DIFF
	cleanMWords();

	QString file1 = caption();
	if (file1.isNull() || file1.isEmpty() || file1 == "*" || file1 == "empty buffer") {
		QMessageBox::warning(0,tr("Warning"), tr("Session diff not supported on new files"));
		return;
	}
	// try to save the file
	if (!save()) {
		QMessageBox::warning(0, tr("Error"), tr("Session Diff operation aborted!"));
		return;
	}
	file1 = caption();

	// get file from backup
	QFileInfo qf(file1);
	char *f2 = Global::getBackupFname((const char *) qf.fileName());
	QString file2 = QString(f2);
	delete[] f2;
LOG("session diff file1: %s", (const char *) file1);
LOG("session diff file2: %s", (const char *) file2);

	QFont fnt = e_->font();
	if (file1.isEmpty() || file1.isNull() || file2.isEmpty() || file2.isNull())
		return;
	DiffWorksDialog *wd = new DiffWorksDialog(fnt, file2, file1);
	wd->setCaption(tr("session diff"));
	wd->exec();
	delete wd;
#endif
}

void Editor::beautify() {
	BeautyDialog *be = new BeautyDialog();

	be->setCaption(tr("Beautify"));
	if (!be->exec()) {
		delete be;
		return;
	}

	QString file = caption();
	if (file.isNull() || file.isEmpty() || file == "*") {
		QMessageBox::warning(0,tr("Warning"), tr("First you will need to save the file"));
		return;
	}

	// save file
	if (!save()) {
		QMessageBox::warning(this,tr("Warning"),
			QString(tr("The operation will not take place.\n\n")));
		return;
	}

	file = caption();

	tw_user_->setText("");
	BCursor busy;
	msgScreen(tr("robcpp, please wait..."));
	QString qs = be->getCmd(file);
	tw_user_->setText(qs);
	tw_->setCurrentPage(ID_USER);
	char *result = UTILS::run_prg_malloc((const char *) qs);
	if (result == 0) {
		QMessageBox::warning(0, tr("Warning"), tr("robcpp failed!"));
		// force a resave of the file
		save();
		return;
	}

	msgScreen(tr("robcpp done"), 2000);
	e_->setText(QString::fromUtf8(result));
	e_->setFocus();
	delete[] result;
}

void Editor::htmltidy() {
	QString file = caption();
	if (file.isNull() || file.isEmpty() || file == "*") {
		QMessageBox::warning(0,tr("Warning"), tr("First you will need to save the file"));
		return;
	}

	// save file
	if (!save()) {
		QMessageBox::warning(this,tr("Warning"),
			QString(tr("The operation will not take place.\n\n")));
		return;
	}

	file = caption();
	QString cmd;
	if (file.endsWith(".html") || file.endsWith(".htm"))
		cmd = "rotidy -q -utf8 -o ~/.rospell/backup/tidyoutput " + file + " 2>&1";
	else if (file.endsWith(".xml"))
		cmd = "roxml " + file + " 2>&1";
	else {
		QMessageBox::warning(this,tr("Warning"),
			QString(tr("Supporting only .html, .htm, and .xml files.\nThe operation will not take place.\n\n")));
		return;
	}

	BCursor busy;

	char *result = UTILS::run_prg_malloc((const char *) cmd);
	tw_user_->setText(QString::fromUtf8(result));
	tw_->setCurrentPage(ID_USER);
	msgScreen(tr("syntax check done"), 2000);
	delete[] result;
}

void Editor::kloc() {
	tw_user_->setText("");
	msgScreen(tr("rokloc, please wait..."));
	BCursor busy;

	char *result = UTILS::run_prg_malloc("rokloc -d -s");
	tw_user_->setText(result);
	tw_->setCurrentPage(ID_USER);
	msgScreen(tr("rokloc done"), 2000);
	delete[] result;
}

void Editor::make() {
	tw_user_->setText("");

	QString cmd;
	bool save_all = false;
	if (Global::instance()->make_cmd_.isEmpty()) {
		ProgramDialog *d = new ProgramDialog("make 2>&1", true);
		d->setCaption(tr("make"));
		if (!d->exec()) {
			delete d;
			return;
		}
		cmd = d->getCmd();
		save_all = d->getSaveAll();
		if (d->getDontShow()) {
			Global::instance()->make_cmd_ = cmd;
			Global::instance()->make_save_all_ = save_all;
		}
		delete d;
	}
	else {
		cmd = Global::instance()->make_cmd_;
		save_all = Global::instance()->make_save_all_;
	}
	msgScreen(tr("make, please wait..."));
	tw_->setCurrentPage(ID_USER);
	BCursor busy;
	if (save_all) {
		if (!saveAll()) {
			QMessageBox::warning(0, tr("Warning"), tr("make operation aborted!"));
			return;
		}
	}

	QTUTILS::run_program(cmd, tw_user_, (char *) (const char *) QString(tr("make done")));
	tw_->setCurrentPage(ID_USER);
	msgScreen(tr("make done"), 2000);
}

void Editor::stripTrailingSpaces() {
	BCursor busy;
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString intext = e_->text();
	QString newtext = QTUTILS::stripTrailingSpaces(intext);
	e_->setText(newtext);
	e_->setCursorPosition(para, index, false);
}

void Editor::tabToSpace() {
	BCursor busy;
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString intext = e_->text();
	QString newtext = QTUTILS::tab2space(intext);
	e_->setText(newtext);
	e_->setCursorPosition(para, index, false);
}

void Editor::cedilla2commabelow() {
	BCursor busy;
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString intext = e_->text();
	QString newtext = QTUTILS::cedilla2commabelow(intext);
	e_->setText(newtext);
	e_->setCursorPosition(para, index, false);
}

void Editor::commabelow2cedilla() {
	BCursor busy;
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString intext = e_->text();
	QString newtext = QTUTILS::commabelow2cedilla(intext);
	e_->setText(newtext);
	e_->setCursorPosition(para, index, false);
}

void Editor::toggleSpeller() {
	if (show_speller_) {
		mwords_->hide();
		show_speller_ = false;
	}
	else {
		mwords_->show();
		show_speller_ = true;
	}
}
